/*
 * Copyright(C) Seiko Epson Corporation 2016. All rights reserved.
 *
 * Warranty Disclaimers.
 * You acknowledge and agree that the use of the software is at your own risk.
 * The software is provided "as is" and without any warranty of any kind.
 * Epson and its licensors do not and cannot warrant the performance or results
 * you may obtain by using the software.
 * Epson and its licensors make no warranties, express or implied, as to non-infringement,
 * merchantability or fitness for any particular purpose.
 */

package nmd.emd.epson.com.probt_2000voicecommand;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.media.epson.IVoiceCommandInterface;
import android.media.epson.VoiceCommandClient;
import android.media.epson.VoiceCommandClientCallbacks;
import android.os.Bundle;
import android.os.Environment;
import android.os.RemoteException;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.SeekBar;
import android.widget.TextView;

import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

public class MainActivity extends Activity implements VoiceCommandClientCallbacks {

    private boolean mIsRunningVCS = false;
    private VoiceCommandClient mVoiceCommandClient = null;
    private IVoiceCommandInterface mVoiceCommandSystem;
    private String mLmsCurrentPath;
    private final String mDefaultLmsPath = "/system/vendor/bin/epe_au01_E1.lms";
    private float mSnr = 15;
    private int mAmp = 1500;

    private TextView mTvDefaultCommands1;
    private TextView mTvDefaultCommands2;
    private TextView mTvDefaultCommands3;
    private TextView mTvCustomCommands1;
    private TextView mTvCustomCommands2;
    private TextView mTvCustomCommands3;
    private TextView mHeadsetStatus;
    private TextView mTvDetectedCmd;
    private ImageView mIvAudioWave;
    private ImageButton mImgBtnToggleVCS;
    private RadioButton mRdbDefaultCmd;
    private RadioButton mRdbCustomCmd;

    //A broadcast receiver notifies you when an event happens external to your application
    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {

        /**
         * This method gets executed when the microphone connection state changes
         * @param context Application Context
         * @param intent Headset plug in intent
         */
        @Override
        public void onReceive(Context context, Intent intent) {
            final String action = intent.getAction();
            if (Intent.ACTION_HEADSET_PLUG.equals(action)) {

                int micConnectedState = intent.getIntExtra("state", -1);

                if (micConnectedState == 0) {

                    mHeadsetStatus.setTextColor(Color.RED);

                    mHeadsetStatus.setText(R.string.please_connect_the_headset);

                } else if (micConnectedState == 1) {

                    mHeadsetStatus.setTextColor(Color.BLACK);

                    mHeadsetStatus.setText(R.string.headset_is_connected);
                }
            }
        }
    };

    /**
     * This method get executed when you click on either of the radio buttons,
     * and is used to toggle the radio buttons between being selected or not
     * @param view Selected view
     */
    public void onRadioClick(View view) {

        if (view.getId() == mRdbDefaultCmd.getId()){

            mRdbDefaultCmd.setChecked(true);

            mRdbCustomCmd.setChecked(false);

            mLmsCurrentPath = mDefaultLmsPath;

        } else {

            mRdbDefaultCmd.setChecked(false);

            mRdbCustomCmd.setChecked(true);

            String customLmsPath = Environment.getExternalStorageDirectory() + getString(R.string.voicecommandlms);
            File f = new File(customLmsPath);

            if(f.exists() && !f.isDirectory()) {

                mLmsCurrentPath = customLmsPath;

            } else {

                try {

                    File dir = new File(Environment.getExternalStorageDirectory() + getString(R.string.customlmsdirectory));

                    dir.mkdirs();

                    InputStream in = getAssets().open(getString(R.string.customlmsfilename));

                    File file = new File(customLmsPath);

                    file.createNewFile();

                    FileUtils.copyToFile(in, file);

                    mLmsCurrentPath = customLmsPath;


                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Button on click code to restart the demo and display the splash screen
     */
    public void onClickRestart(View view) {

        Intent i = getBaseContext().getPackageManager()
                .getLaunchIntentForPackage(getBaseContext().getPackageName());

        i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);

        startActivity(i);
    }

    /**
     * This is the method on the onClick call from the image button to start
     * the voice detection service
     */
    public void ToggleVoiceCommand(View view) {

        //1. Is the voice command system already running
        if (!mIsRunningVCS) {

            //2. If it is not then display change the image icon to stop
            mImgBtnToggleVCS.setBackgroundResource(R.drawable.microphone_off);

            try {
                    mVoiceCommandSystem.setSnr(mSnr);

                    mVoiceCommandSystem.setAmp(mAmp);

                    mVoiceCommandSystem.start(mLmsCurrentPath);

                    mRdbDefaultCmd.setEnabled(false);

                    mRdbCustomCmd.setEnabled(false);

            } catch (RemoteException e) {

                e.printStackTrace();
            }

            mImgBtnToggleVCS.setBackgroundResource(R.drawable.microphone_off);

            mIsRunningVCS = true;

        } else if (mIsRunningVCS == true ) {

            try {

                mTvDetectedCmd.setText("");

                mVoiceCommandSystem.stop();

                mImgBtnToggleVCS.setBackgroundResource(R.drawable.microphone_on);

                mIsRunningVCS = false;

                mRdbDefaultCmd.setEnabled(true);

                mRdbCustomCmd.setEnabled(true);

            } catch (RemoteException e) {

                e.printStackTrace();
            }
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);

        mTvDefaultCommands1 = (TextView) findViewById(R.id.tvDefaultCommands1);

        mTvDefaultCommands2 = (TextView) findViewById(R.id.tvDefaultCommands2);

        mTvDefaultCommands3 = (TextView) findViewById(R.id.tvDefaultCommands3);

        mTvCustomCommands1 = (TextView) findViewById(R.id.tvCustomCommands1);

        mTvCustomCommands2 = (TextView) findViewById(R.id.tvCustomCommands2);

        mTvCustomCommands3 = (TextView) findViewById(R.id.tvCustomCommands3);

        mHeadsetStatus = (TextView) findViewById(R.id.tvHeadsetStatus);

        mTvDetectedCmd = (TextView) findViewById(R.id.tvDetectedCmd);

        mIvAudioWave = (ImageView) findViewById(R.id.ivAudioWave);

        mImgBtnToggleVCS = (ImageButton) findViewById(R.id.imgBtnToggleVCS);

        mRdbDefaultCmd = (RadioButton) findViewById(R.id.rdbDefaultCommands);

        mRdbCustomCmd = (RadioButton) findViewById(R.id.rdbCustomCommands);

        mLmsCurrentPath = mDefaultLmsPath;

        initialize();

        mVoiceCommandClient = new VoiceCommandClient();

        mVoiceCommandClient.bindToRemoteRunningService(this);

        SeekBar mSlideNoiseLevel = (SeekBar) findViewById(R.id.slideNoiseLevel);

        mSlideNoiseLevel.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {

                switch (progress){
                    case 0:
                        mSnr = 10;
                        mAmp = 1000;
                        break;
                    case 1:
                        mSnr = 15;
                        mAmp = 1500;
                        break;
                    case 2:
                        mSnr = 15;
                        mAmp = 2500;
                        break;
                    default:
                        mSnr = 15;
                        mAmp = 1500;
                        break;
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();

        mVoiceCommandClient.registerCallback(this);

        IntentFilter filter = new IntentFilter(Intent.ACTION_HEADSET_PLUG);

        getApplicationContext().registerReceiver(mReceiver, filter);

    }

    @Override
    protected void onPause() {
        super.onPause();

        mVoiceCommandClient.unRegisterCallback();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();

        mVoiceCommandClient.unBindFromRemoteRunningService(this);
    }

    @Override
    public void onServiceConnected() {

        mVoiceCommandSystem = mVoiceCommandClient.getVoiceCommandSystemInterface();
    }

    @Override
    public void onServiceDisconnected() {

    }

    @Override
    public void onSpeakable(boolean speakable) {

        if (speakable) {

            mIvAudioWave.setImageResource(R.drawable.audio_wave_green);

        } else {

            mIvAudioWave.setImageResource(R.drawable.audio_wave_red);

        }
    }

    @Override
    public void onVoiceCommand(int id, String word) {

        mTvDetectedCmd.setText(word);

    }

    /**
     * Helper method to set up the commands
     */
    private void initialize(){

        String[] defaultCommands = new String[]{"Menu", "Enter", "Next", "Return", "Action",
                "Action 1", "Action 2", "Red", "Yellow", "Blue", "Green", "Zoom In",
                "Zoom Out", "Start", "Stop", "Version", "Display On", "Display Off"
        };

        loadCommands(defaultCommands, CommandType.DefaultCommand);

        String[] customCommands = new String[]{"John Smith", "Jennifer Jones", "Frankfurt",
                "Paris", "Singapore", "Wind Turbine", "Factory", "Workshop", "Marine Zone",
                "Power Plant", "Remote Assistance", "Maintenance", "Training", "Hard Shutdown",
                "Lock 1", "Lock 2", "Twist Off", "Drill"};

        loadCommands(customCommands, CommandType.CustomCommand);
    }

    /**
     * Helpeer method to dsiplay the commands correctly on the UI
     * @param commands list of spoken commands
     * @param commandType Is it a built in default command or custom created
     */
    private void loadCommands(String[] commands, CommandType commandType) {

        int paneCounter = 1;

        for (String defaultCommand : commands){

            if (paneCounter <=6) {

                if (commandType == CommandType.DefaultCommand) {

                    mTvDefaultCommands1.append(defaultCommand + "\n");

                }else if (commandType == CommandType.CustomCommand) {

                    mTvCustomCommands1.append(defaultCommand + "\n");
                }

            } else if (paneCounter <=12) {

                if (commandType == CommandType.DefaultCommand) {

                    mTvDefaultCommands2.append(defaultCommand + "\n");

                }else if (commandType == CommandType.CustomCommand) {

                    mTvCustomCommands2.append(defaultCommand + "\n");
                }

            } else {

                if (commandType == CommandType.DefaultCommand) {

                    mTvDefaultCommands3.append(defaultCommand + "\n");

                }else if (commandType == CommandType.CustomCommand) {

                    mTvCustomCommands3.append(defaultCommand + "\n");
                }
            }

            paneCounter++;
        }
    }
}
